/*******************************************************************************
Project: Gender, real estate, and property tax compliance

Name   : 0_master.do  

Task   : Running every do file reproducing results for the analysis of the paper:
"Exploring the Gender Divide in Real Estate Ownership and Property Tax Compliance"
		 
Authors: Tatiana Flores, Dario Tortarolo, Jose Carlo Bermúdez, Luis Schiavoni,
		 Thiago Scot, Guillermo Cruces.
*******************************************************************************/

	version 18
	clear all
	set more off
	
	* Start timer
	timer clear 1
	timer on 1

/*******************************************************************************
				Working paths, globals, and packages
*******************************************************************************/
	
* Setting working paths for specific users
	*Dario Tortarolo
	if "`c(username)'" == "Dario" {	
		global path "D:/Dropbox/3dF_Gender/"
	}
	else if "`c(username)'" == "wb515436" {
		global path "C:/Users/wb515436/Dropbox/3dF_Gender/"
	}
	*Jose Carlo
	else if "`c(username)'" == "bermu" {	
		global path "C:/Users/bermu/Dropbox/3F_Gender"
		global do "C:/Users/bermu/OneDrive/Escritorio/property-taxes-3df/1_Scripts (FOR REPLICATION)"
	}
	*Tatiana
	else if "`c(username)'" == "wb596027" {	
		global path "C:/Users/wb596027/Dropbox/3F_Gender"
	}
	*Thiago
	else if "`c(username)'" == "tscot" {	
		global path "/Users/tscot/Dropbox/3F_Gender_Reproducibility"
		global do "/Users/tscot/Library/CloudStorage/OneDrive-WBG/Documents/GitHub/property-taxes-3df/1_Scripts (FOR REPLICATION)"															// <-- Insert the location of the code
	}
	
	* Reproducibility
	else if "`c(username)'" == "wb558768" {	
		global path "C:\Users\wb558768\Downloads\rep-checks\263\v1"															// <-- Change user path here
		global do "C:\Users\wb558768\Downloads\rep-checks\263\v1\property-taxes-3df-main\1_Scripts (FOR REPLICATION)"															// <-- Insert the location of the code
	}

* Setting specific folder paths 
	global raw_data 	  "${path}/Data/Raw"
	global processed_data "${path}/Data/Processed"
	global figures 		  "${path}/Figures"
	global tables  		  "${path}/Tables"
	
	
* Global for figures appearance
	if c(version) == 18.0 {
	global graphopt "legend(region(lcolor(none))) graphr(color(white)) ylabel(, nogrid) scheme(s2color)"
	}
	else {
	global graphopt "legend(region(lcolor(none))) graphr(color(white)) ylabel(, nogrid)"
	}
	
	global bgopt "plotregion(icolor(white)) graphregion(fcolor(white) color(white)) bgcolor(white)"
	
	sysdir set PLUS "${do}/ado"

* Install package, if required
	global packs estout matchit freqindex geoplot moremata geoframe binscatter cdfplot palettes colrspace egenmore winsor2 reghdfe
	foreach p of global packs {
		cap which `p'
		if (_rc) ssc install `p'
	}	
	
	local github "https://raw.githubusercontent.com"
*net install gtools, from(`github'/mcaceresb/stata-gtools/master/build/)
*Install frmttable, not on ssc
*net install sg97_5, from(http://www.stata-journal.com/software/sj12-4)
	
	
/*******************************************************************************
									Running Do Files
*******************************************************************************/


************************** 1. Cleaning/Harmonize Data 
	
	*** Prepare working database 
	* This do file is not available for reproducibility because it was run by the staff at 3dF Municipality under our guidance.
	*do "$do/1a_prepare_db.do"

	*** Harmonize and aggregate outcomes
	do "$do/1b_prepare_variables.do"
	
	
************************** 2. Descriptive Analysis

	*** Tables with descriptive statistics
	do "$do/2a_analysis_tables.do"
	
	*** Figures with descriptive analysis
	do "$do/2b_analysis_figures.do"

	
************************** 3. RCT Analysis

	*** Balance tests
	do "$do/2c_balance_checks.do"
	
	*** Figures from RCT analysis
	do "$do/2d_analysis_RCT.do"
	do "$do/2e_analysis_cuota10.do"
	
	
************************** 4. Maps Visualization 
	
	*** Maps
	do "$do/3a_maps.do"
	do "$do/3b_maps_seccion_cadastro.do"	
	

	
/*******************************************************************************
					Time the do file takes to be run
*******************************************************************************/	
	qui{
		timer off 1
		
		timer list 1
		local minutes1 = round(`r(t1)'/60)
	}
	
	dis in red "The entire code took `minutes1' minutes to be run."
	
	
