/*******************************************************************************
Name   : 3a_maps.do

Task   : Maps at the block level.

Input  : ${raw_data}/3df_manzana.dta
		 ${raw_data}/cadastro.dta
		 ${raw_data}/3df_manzana_shp
		 
Output : ${processed_data}/3df_manzana_new.dta
		 ${processed_data}/manzana_level_data_map.dta
		 map_mzn_res_property_number_arbavaluation-2021.png
*******************************************************************************/



***************************************************************************************
***********************map at the manzana level ***************************************
*preparing shapefiles 
use "${raw_data}/3df_manzana.dta", clear

	gen mzl = substr(CCA, -1, 1)
	
	gen mz_zeros = substr(CCA, -6, 3)
	
	replace mzl ="." if mzl =="0"
save "${processed_data}/3df_manzana_new.dta", replace

*******************************************************
* preapring cadastro data 
use "${raw_data}/cadastro.dta", clear
	gen mz_zeros = string(mz, "%03.0f")
	
	replace mzl ="." if mzl ==""
	replace mzl ="." if mzl =="  "
	replace mzl ="." if mzl ==" "
	replace mzl ="." if mzl =="0"
	
	gen mzl_new = substr(mzl, 1, 1)
	drop mzl
	rename mzl_new mzl
	
	*counting properties
	gen cuenta_n = _n
	
	* valuacion in thousands pesos
	gen valuacion_th = valuacion/1000
	format (valuacion_th) %9.0f
	

	*residential only
	keep if categoria==1
	
	*keep if inrange( valuacion_th , 0 , r(p99) )
	
	/* checking percentiles values at the property level: 

----------------------------------------------------------------------------------------
valuacion_th                                                                 (unlabeled)
----------------------------------------------------------------------------------------

                  Type: Numeric (float)

                 Range: [1.833,644894.63]             Units: .001
         Unique values: 89,585                    Missing .: 7,708/118,830

                  Mean: 1096.17
             Std. dev.: 2405.52

           Percentiles:     10%       25%       50%       75%       90%
                        323.157   561.973   848.061   1347.57   2061.15

	*/
	

*collapse at the sec circ level
	gcollapse (count) cuenta_n (mean) valuacion valuacion_th, by(mzl mz_zeros)

*merging with shapefile 
	merge m:m mzl mz_zeros using "${processed_data}/3df_manzana_new.dta"
	
	/*
	
    Result                      Number of obs
    -----------------------------------------
    Not matched                            72
        from master                        61  (_merge==1)
        from using                         11  (_merge==2)

    Matched                             2,897  (_merge==3)
    -----------------------------------------

	*/
	
	*keep only matched
	keep if _merge==3
	
	drop _merge
	
save "${processed_data}/manzana_level_data_map.dta", replace
	
* Load shapefile data
geoframe create geo1 "${processed_data}/manzana_level_data_map.dta", replace shp("${raw_data}/3df_manzana_shp")

*calculating quantiles
centile cuenta_n, centile(20 40 60 80 100)

*map for number of properties 
	geoplot ///
    (area geo1 cuenta_n, level(5, quantiles) color(reds) label("@lb - @ub (N=@n)")) ///
    , angle(45) legend(size(3.5) order(1 "Quantile 1 (116)" 2 "Quantile 2 (715)" 3 "Quantile 3 (926)" 4 "Quantile 4 (1140)" 5 "Quantile 5 (4436)")) 
	graph export "${figures}/map_mzn_res_property_number_arbavaluation-2021.png", replace

	