/*******************************************************************************
Name   : 3b_maps_seccion_cadastro.do 

Task   : Plot maps for the properties number and average values at the seccion 
		 cadastral and manzana level
		 
Input  : ${raw_data}/3df.dta
		 ${processed_data}/working_data_3dF_yearly.dta
		 ${raw_data}/cadastro.dta
		 ${raw_data}/3df_shp
		 
Output : ${processed_data}/3df_new.dta
		 ${processed_data}/cadaster_seccion_level_data_map.dta
		 map_seccion_catastral_properties_compliance.png	
*******************************************************************************/


********************************************************************************************
***********************map at the seccion catastral level **********************************

use "${raw_data}/3df.dta", clear

	gen  circ1= substr(CCA,5,1) 

	gen  sec= substr(CCA,7,1) 

	destring circ1, generate(circ)

	replace sec = "LL" if _ID == 7
	
	encode sec, gen(sec_d)
	
	*gen cir_sec = circ + sec_d
	
	*list _ID sec if _ID == 7

save "${processed_data}/3df_new.dta", replace
	
	
*********************************************************
	* preapring admin data 
	
	
	use "${processed_data}/working_data_3dF_yearly.dta", clear
		*gen sometime_payer = (always_payer == 0 & never_payer == 0)
	
	gen cuenta = substr(nro_cuenta, 1, 6)
	
	keep if anio==2019
	
	*mwrging at the cuenta level
	merge m:1 cuenta using "${raw_data}/cadastro.dta"
	
	
	*fixing erorr for sec 0o
	replace sec = "O" if sec == "0O"
	
	*gen  sec_1= substr(sec,1,1) 
	*replace sec_1="LL" if sec=="LL"
	
	gen sec_1 = substr(sec, 1, 1)
	replace sec_1 = "LL" if sec == "LL" | sec == "0O"
	
	*counting properties
	gen cuenta_n = _N
	
	* valuacion in thousands pesos
	gen valuacion_th = valuacion/1000
	
	*p99 only
	*keep if inrange( valuacion_th , 0 , r(p99) )
	
	gen sometime_payer = (never_payer==0 & always_payer==0)
	
	*collapse at the sec circ level
	gcollapse (count) cuenta_n (mean) never_payer always_payer sometime_payer valuacion valuacion_th, by(sec_1 circ)
	
	label variable never_payer "Never Payer"
	label variable always_payer "Always Payer"
	label variable sometime_payer "Sometime Payer"
		
	*check circ+secc that has only 1 property and outlier property value (Q-4, 0-6, 2-4)
	*drop if cuenta_n <= 1
	rename sec_1 sec
	
	*I need to check what to do for seccions that contain only 1 property
	replace valuacion = . if cuenta_n <=1

	*merging with shapefile 
	merge m:1 circ sec using "${processed_data}/3df_new.dta"

	*keep only matched
	keep if _merge==3
	
	drop _merge
	
	save "${processed_data}/cadaster_seccion_level_data_map.dta", replace
	
	
**********************************************************
*rotate
* Plot the map

* Load shapefile data
geoframe create nuts3 "${processed_data}/cadaster_seccion_level_data_map.dta", replace shp("${raw_data}/3df_shp")


	geoplot ///
    (area nuts3 valuacion_th, level(5, quantile) color(blues))  ///
    (pie nuts3 never_payer sometime_payer always_payer, size(*0.5) explode(2 = 30) angle(0) ///
    color(red yellow green) label(, reverse)) ///
    , angle(45) legend( size(3.4) layout (- "Assessed Value" 1 | - "Compliance" 2) position(ne) order(1 "Quantile 1" 2 "Quantile 2" 3 "Quantile 3" 4 "Quantile 4" 5 "Quantile 5" ///
	6 "Never Payer" 7 "Sometime Payer" 8 "Always Payer")) ///
    graphregion(margin(small)) title("")
graph export "${figures}/map_seccion_catastral_properties_compliance.png", replace
	
